# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-01-12 03:18
from __future__ import unicode_literals

from django.db import migrations, models
import swapper
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        swapper.dependency('mark', 'MarkdownType'),
    ]

    operations = [
        migrations.CreateModel(
            name='MarkdownType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
            ],
            options={
                'abstract': False,
                'swappable': swapper.swappable_setting('mark', 'MarkdownType'),
                'db_table': 'wq_markdowntype',
            },
        ),
        migrations.CreateModel(
            name='Markdown',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('summary', models.CharField(blank=True, max_length=255, null=True)),
                ('markdown', models.TextField(blank=True, null=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=swapper.get_model_name('mark', 'MarkdownType'))),
            ],
            options={
                'abstract': False,
                'db_table': 'wq_markdown',
            },
        ),
    ]
